/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "actionmanager.h"
#include "utils.h"

// The constructor of the action manager sets all of the variables
// to their defaults and adds a NULL action to the actions list so
// that the actionmanager will know when there are no more undo(s)
action_manager::action_manager()
{
  current_position = 0;
  current_size = 1;
  current_buffer_offset = -1;
  
  // Create a null action & push it back
  action temp;
  temp.text = "NULL";
  actions.push_back(temp);
}

// Add a new action to the actionmanager. This function receives whether the
// text is being added or removed from the buffer, the offset in characters
// of the current addition/removal and the string that contains those characters
void action_manager::add_new_action(bool add_text, int chars, Glib::ustring str)
{
  // Remove all actions after the current point if adding a new one
  if (current_position != current_size-1)
  {
    while (current_size-1 > current_position) 
    {
      actions.pop_back();
      current_size--;
    }
  }
  
  // Limit the actions to 50 by erasing the first one if current size is 50
  if (current_size >= 50)
  {
    vector<action>::iterator start = actions.begin();
    actions.erase(++start);
  }
  // Increment counters if there aren't 50 actions yet
  else 
  {
    current_position++;
    current_size++;
  }
  
  // Create temporary action
  action temp;
  temp.text_was_added = add_text;
  temp.offset = chars;
  temp.text = str;
  
  // Add the action to the list
  actions.push_back(temp);
  current_buffer_offset = chars;
}

// As explained by the function name, this removes all of the actions from
// the actions list and replaces them will the original NULL action
void action_manager::erase()
{
  // Erase all actions from the buffer
  actions.erase(actions.begin(), actions.end());
  current_position = 0;
  current_size = 1;
  
  // Re-create the null action
  action temp;
  temp.text = "NULL";
  actions.push_back(temp);
}

// The last action is undone. It receives the position of the last addition
// or removal and whether the text was added or removed. Both of these are
// passed as reference, so they are returned. The text from the undo is also
// returned. A NOACTION string is returned if there are no actions left to undo.
// The current position of the actionmanager is moved back one to allow for redo(s)
Glib::ustring action_manager::undo(int& chars, bool& add_text)
{
  // Check to make sure it is not the limit of undo's
  if (actions[current_position].text != "NULL")
  {
    // Return text offset and whether it was added or deleted
    chars = actions[current_position].offset;
    add_text = actions[current_position].text_was_added;
    
    // Return the text
    Glib::ustring txt = actions[current_position].text;
    current_position--;
    return txt;
  }
  
  return "NOACTION";
}

// The last action is redone. It receives the position of the last addition
// or removal and whether the text was added or removed. Both of these are
// passed as reference, so they are returned. The text from the redo is also
// returned. A NOACTION string is returned if there are no actions left to redo.
// The current position of the actionmanager is moved forward one to allow 
// for undo(s)
Glib::ustring action_manager::redo(int& chars, bool& add_text)
{
  // Check to make sure there is something to redo
  if (current_position < current_size-1)
  {
    // Return offset and whether text was added or deleted
    current_position++;
    chars = actions[current_position].offset;
    add_text = actions[current_position].text_was_added;
    
    // Return the text
    Glib::ustring txt = actions[current_position].text;
    return txt;
  }
  
  return "NOACTION";
}

// This returns whether the current action is removing or adding text. This
// allows the main class to append concurrent actions such as multiple keys
// added or deleted put into one action
bool action_manager::get_current_action_add_or_del()
{
  return actions.back().text_was_added;
}

// Return the current offset of the buffer
int action_manager::get_current_action_offset()
{
  return current_buffer_offset;
}

// Add the text to the end of the current action to allow for concurrent actions
// of the same type to be one undo or redo. This function receives the offset
// in characters of the cursor and the string of text that is added or removed
void action_manager::add_to_end_current_action(bool move_offset, Glib::ustring str)
{
  actions.back().text += str;
  if (move_offset)
  {
    current_buffer_offset++;
    actions.back().offset++;
  }
}

// Add the text to the end of the current action to allow for concurrent actions
// of the backspaces to be one undo or redo. This function receives the offset
// in characters of the cursor and the string of text that is added or removed
void action_manager::add_to_begin_current_action(Glib::ustring str)
{
  actions.back().text.insert(actions.back().text.begin(), str[0]);
  current_buffer_offset--;
  actions.back().offset--;
}
