/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "edit.h"
#include "utils.h"
#include "gotoline.h"
#include "syntax.h"
#include "keypress.h"

// Cut the currently selected text from the buffer. Then add the text to the
// actionmanager of the current buffer and redo the syntax highlighting
void openldev::on_edit_cut()
{
  Glib::RefPtr<Gtk::Clipboard> Clipboard = Gtk::Clipboard::get();
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = noteman->get_current_textview()->get_buffer();
  flip_insert_selection_bounds(text_buffer);
  
  screen.set_clipboard(text_buffer->get_text(text_buffer->get_insert()->get_iter(),
    text_buffer->get_selection_bound()->get_iter(), true));
  noteman->get_aman().add_new_action(false, 
                text_buffer->get_insert()->get_iter().get_offset(), 
                screen.get_clipboard());
  
  text_buffer->cut_clipboard(Clipboard, true);
  set_syntax(text_buffer, false, true, env);
  noteman->mark_current_tab_modified(true); 
  noteman->scroll_to_mark();
  noteman->update_line_numbers();
}

// Cut the currently selected text from the buffer.
void openldev::on_edit_copy()
{
  Glib::RefPtr<Gtk::Clipboard> Clipboard = Gtk::Clipboard::get();
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = noteman->get_current_textview()->get_buffer();
  screen.set_clipboard(text_buffer->get_text(text_buffer->get_insert()->get_iter(),
    text_buffer->get_selection_bound()->get_iter(), true)); 
  text_buffer->copy_clipboard(Clipboard);
}

// Paste the clipboard text to the current cursor position. Then add the text 
// to the actionmanager of the current buffer and redo the syntax highlighting
void openldev::on_edit_paste()
{
  Glib::RefPtr<Gtk::Clipboard> Clipboard = Gtk::Clipboard::get();
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = noteman->get_current_textview()->get_buffer();
  
  noteman->get_aman().add_new_action(true, 
       text_buffer->get_insert()->get_iter().get_offset()+screen.get_clipboard().length(), 
       screen.get_clipboard());
  
  text_buffer->paste_clipboard(Clipboard, true);
  set_syntax(text_buffer, false, true, env);
  noteman->mark_current_tab_modified(true); 
  noteman->scroll_to_mark();
  noteman->update_line_numbers();
}

// Select all text in the text_view
void openldev::on_edit_selectall() 
{
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = noteman->get_current_textview()->get_buffer();
  text_buffer->move_mark(text_buffer->get_insert(), text_buffer->begin());
  text_buffer->move_mark(text_buffer->get_selection_bound(), text_buffer->end());
}

// Deselect any text that is selected
void openldev::on_edit_deselect() 
{
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = noteman->get_current_textview()->get_buffer();
  text_buffer->move_mark(text_buffer->get_selection_bound(), text_buffer->get_insert()->get_iter());
}

// Undo the last action that was performed. If there was no action, then don't
// do anything. To undo a delete, just insert the text. To undo an insert, the
// text must be removed from the specified offset & then backwards the length
void openldev::on_edit_undo() 
{
  int offset;
  bool add_text;
  Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
  Gtk::TextIter iter;
  
  // Get the undo information and get the iter at the offset
  Glib::ustring text = noteman->get_aman().undo(offset, add_text);
  iter = buffer->get_iter_at_offset(offset);
  
  // If there was an action to undo...
  if (text != "NOACTION")
  {
    // Delete the text if it was added
    if (add_text) 
    {
      Gtk::TextIter begin = iter;
      begin.backward_chars(text.length());     
      buffer->erase(begin, iter);
    }
    // Add the text if it was deleted
    else 
    {
      buffer->insert(iter, text);
    }
    
    // Update the line numbering and then the syntax highlighting
    set_syntax(buffer, false, true, env);
    noteman->mark_current_tab_modified(true); 
    noteman->scroll_to_mark();
    noteman->update_line_numbers();
  }
}

// Redo the last action that was undone. If there was no action, then don't
// do anything. To redo an insert, you must remove the text from the specified
// offset to backwards the length of the text. To redo a delete, just move
// backward the length of the string & insert it
void openldev::on_edit_redo()  
{ 
  int offset;
  bool add_text;
  Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
  Gtk::TextIter iter;
  
  // Get the action to undo and move the iter to the offset
  Glib::ustring text = noteman->get_aman().redo(offset, add_text);
  iter = buffer->get_iter_at_offset(offset);
  
  // If there is an action to redo...
  if (text != "NOACTION")
  {
    // Add the text if it was deleted by undo
    if (add_text) 
    {
      iter.backward_chars(text.length());
      buffer->insert(iter, text);
    }
    // Delete text if it was added by undo
    else 
    {
      Gtk::TextIter end = iter;
      end.forward_chars(text.length());  
      buffer->erase(iter, end);
    }
    
    // Update the line numbering & then the syntax highlighting
    set_syntax(buffer, false, true, env);
    noteman->mark_current_tab_modified(true); 
    noteman->scroll_to_mark();
    noteman->update_line_numbers();
  }
}
