/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "filemanager.h"
#include "utils.h"

#include <gtkmm/liststore.h>
#include <gtkmm/fileselection.h>
#include <gtkmm/window.h>
#include <glib.h>

// The constructor for the File Manager receives the strings for the source
// files, header files, other files, and the project location. It initializes
// the widgets and sets their properties. The dialog is then created and shown
// and the signals are connected to their widgets
file_manager::file_manager(Glib::ustring src, Glib::ustring hdr, 
                           Glib::ustring othr, Glib::ustring loc)
{
  // Setting source, header & other files and the location
  source_files = src;
  header_files = hdr;
  other_files = othr;
  src_location = Glib::ustring(loc + "src/");
  
  initialize_objects();  
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  dialog->set_default_icon_from_file(string(INSTALL_PREFIX) + "/share/pixmaps/openldev/openldev24.png");
  
  // Populate scrolled_win
  scrolled_win->set_size_request(275,210);
  sources->set_active(true);
  scrolled_win->add(*files);
  scrolled_win->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS); 
  
  // Place widgets on the HBox box
  hbox2->pack_start(*sources);
  hbox2->pack_start(*headers);
  hbox2->pack_start(*others);
  
  // Set the button Gtk::HBox
  close->set_flags(Gtk::CAN_DEFAULT);
  hbox->set_spacing(10);  
  hbox->pack_start(*add, Gtk::PACK_EXPAND_WIDGET);
  hbox->pack_start(*del, Gtk::PACK_EXPAND_WIDGET);
  hbox->pack_start(*close, Gtk::PACK_EXPAND_WIDGET);
  
  // Pack the dialog
  vbox->set_spacing(10);
  vbox->set_border_width(10);
  vbox->pack_start(*hbox2, Gtk::PACK_SHRINK);
  vbox->pack_start(*scrolled_win, Gtk::PACK_EXPAND_WIDGET);
  vbox->pack_start(*hseparator, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK);
  
  // Set dialog properties
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*vbox);
  dialog->set_title("Manage Project Files");
  dialog->set_modal(true);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(false);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(false);
  dialog->set_default(*close);
  
  // Populate sources box & attach signals to the widgets
  on_radio_button_selected();
  sources->signal_clicked().connect(sigc::mem_fun(*this, &file_manager::on_radio_button_selected));
  others->signal_clicked().connect(sigc::mem_fun(*this, &file_manager::on_radio_button_selected));
  add->signal_clicked().connect(sigc::mem_fun(*this, &file_manager::on_add));
  del->signal_clicked().connect(sigc::mem_fun(*this, &file_manager::on_delete));
  close->signal_clicked().connect(sigc::mem_fun(*this, &file_manager::on_close));
 
  dialog->show_all_children();
  dialog->show();
}

// This destructor goes through all of the initialized widgets
// and deletes them to free up memory on close of the dialog
file_manager::~file_manager()
{
  delete add;
  delete del;
  delete close;
  delete files;
  delete sources;
  delete headers;
  delete others;
  delete vbox;
  delete hbox;
  delete hseparator;
  delete scrolled_win;
}

// One or more sources are added to the current type. The filenames(s) are then
// parsed into their base form and added to the list. The sources are then
// sorted and re-populated
void file_manager::on_add()
{
  Gtk::FileChooserDialog dialog("Choose File", Gtk::FILE_CHOOSER_ACTION_OPEN);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::ADD, Gtk::RESPONSE_OK);
  dialog.set_current_folder(src_location);
  dialog.set_select_multiple(true);
  
  int result = dialog.run();
  switch (result)
  {
    // If OK is pressed
    case(Gtk::RESPONSE_OK):
    {
      int location;
      vector<Glib::ustring> filenames = dialog.get_filenames();
      
      // Go through the files & cut them into only the short filenames
      for (unsigned int i=0; i<filenames.size(); i++)
      {
        location = filenames[i].find_last_of("/");
        while (location >= 0)
        {
          filenames[i].erase(0, location+1);
          location = filenames[i].find_last_of("/");
        }
        filenames[i] += "/";
      }
      
      // Add source, header or other file to the list
      for (unsigned int i=0; i<filenames.size(); i++)
      {
        if (sources->get_active())
          source_files += filenames[i];
        else if (headers->get_active())
          header_files += filenames[i];
        else
          other_files += filenames[i];
      }
      
      // Sort the list alphabetically & refresh it
      sort_source_files();
      on_radio_button_selected();
      break;
    }
    default:
      break; 
  }
}

// The currently selected file is deleted from the current TreeView. The
// list is then re-populated
void file_manager::on_delete()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = files->get_selection();
  Gtk::ListStore::iterator iter = selection->get_selected();
  
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    int start;
    row->get_value(0, fn);
    
    // Delete the chosen source, header or other file's name
    if (sources->get_active())
    {
      start = source_files.find(fn, 0);
      source_files.erase(start, fn.length()+1);
    }
    else if (headers->get_active())
    {
      start = header_files.find(fn, 0);
      header_files.erase(start, fn.length()+1);
    }
    else
    {
      start = other_files.find(fn, 0);
      other_files.erase(start, fn.length()+1);
    }
  }
  
  // Refresh the list
  on_radio_button_selected();
}

// On the signal of the close button being clicked, a close response is emitted
void file_manager::on_close()
{
  // Write files to temp files to be parsed
  dialog->response(Gtk::RESPONSE_CLOSE);
}

// The list of sources, headers or other files is refreshed by this function.
// The sources are first removed from the list and then the new list is created.
void file_manager::on_radio_button_selected()
{
  Glib::ustring text;
  int start = 0;
  ModelColumns columns;
  Glib::RefPtr<Gtk::ListStore> model;
  Gtk::TreeModel::Row row;
  
  // Refresh files list
  files->remove_all_columns();
  model = Gtk::ListStore::create(columns);
  files->set_model(model);
  files->append_column("Project Files", columns.col_text);

  // Get whether to list sources for headers
  if (sources->get_active())
    text = source_files;
  else if (headers->get_active())
    text = header_files;
  else
    text = other_files;
  
  // Add the choice of files to the list
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      row = *(model->append());
      row[columns.col_text] = text.substr(start, i-start);
      start = i+1;
    }
  }
}

// This takes the sources, headers and other files and sorts them alphabetically
// by parsing them into a vector, sorting them and then unparsing them.
void file_manager::sort_source_files()
{
  // Loop headers & sources
  for (unsigned int i=0; i<3; i++)
  {
    vector<Glib::ustring> names;
    Glib::ustring temp, str;
    int location;
    
    // Set the source, header or other file's names to the temp string
    if (i == 0) 
    { 
      temp = source_files;
      source_files = "\0";
    } 
    else if (i == 1) 
    {
      temp = header_files;
      header_files = "\0";
    }
    else
    {
      temp = other_files;
      other_files = "\0";
    }
    
    // Parse the files list
    location = temp.find("/",0);
    while (location >= 0) 
    {
      names.push_back(temp.substr(0,location));
      temp.erase(0,location+1);
      location = temp.find("/",0);
    }
    
    // Sort files
    vector<Glib::ustring>::iterator start = names.begin();
    vector<Glib::ustring>::iterator end = names.end();
    sort(start,end);
    
    // Un-parse files list
    for (unsigned int x=0; x<names.size(); x++) 
    {
      if (i == 0)      source_files += names.at(x) + "/";
      else if (i == 1) header_files += names.at(x) + "/";
      else             other_files  += names.at(x) + "/";
    }     
  }
}

// All of the widgets for the File Manager dialog are initialized here
void file_manager::initialize_objects()
{
  dialog = this;
  add = new Gtk::Button(Gtk::StockID("gtk-add"));
  del = new Gtk::Button(Gtk::StockID("gtk-delete"));
  close = new Gtk::Button(Gtk::StockID("gtk-close"));
  files = new Gtk::TreeView();
  hseparator = new Gtk::HSeparator();
  vbox = new Gtk::VBox(false, 0);
  hbox = new Gtk::HBox(false, 0);
  hbox2 = new Gtk::HBox(false, 0);
  scrolled_win = new Gtk::ScrolledWindow();
  
  Gtk::RadioButton::Group _RadioBGroup;
  sources = new Gtk::RadioButton(_RadioBGroup, "Sources");
  headers = new Gtk::RadioButton(_RadioBGroup, "Headers");
  others  = new Gtk::RadioButton(_RadioBGroup, "Others");
}

// This allows the main openldev class to have access to the new lists off
// sources, headers and other files.
Glib::ustring file_manager::get_source_files() { return source_files; }
Glib::ustring file_manager::get_header_files() { return header_files; }
Glib::ustring file_manager::get_other_files()  { return other_files; }
