/**************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "keypress.h"
#include "utils.h"

// Flip the insert and selection bounds so insert is located first. This
// allows for accurate storing of undo and redo data.
void flip_insert_selection_bounds (Glib::RefPtr<Gtk::TextBuffer> buffer)
{
  if (buffer->get_selection_bound()->get_iter().get_offset() <
      buffer->get_insert()->get_iter().get_offset())
  {
    Gtk::TextIter end = buffer->get_selection_bound()->get_iter();
    buffer->move_mark_by_name("selection_bound", buffer->get_insert()->get_iter());
    buffer->move_mark_by_name("insert", end);
  }
}

// If the keypress is a letter or number or symbol, add the action or append
// it to the current action. This function receives the current text buffer
// the beginning iterator of the insert, the key that was pressed, the action
// manager and the environment settings object.
bool key_pressed_alnum (Glib::RefPtr<Gtk::TextBuffer> buffer, Gtk::TextIter begin, 
                        GdkEventKey *key, action_manager& aman, environment_settings env)
{
  Gtk::TextIter end;
  bool selected_text = false;
  
  // Get default iter's and flip the bounds
  flip_insert_selection_bounds(buffer);
  begin = buffer->get_insert()->get_iter();
  end = buffer->get_selection_bound()->get_iter();
  
  // Check if there is selected text
  if (begin != end) 
  {
    aman.add_new_action(false, begin.get_offset(), buffer->get_text(begin, end, true));
    buffer->erase(begin, end);
    selected_text = true;
  }
  
  // Insert text and either update current undo action or add a new one
  buffer->insert_at_cursor(key->string);
  if (!selected_text && aman.get_current_action_add_or_del() &&
      aman.get_current_action_offset()+1 == buffer->get_insert()->get_iter().get_offset())
    aman.add_to_end_current_action(true, key->string);
  else
    aman.add_new_action(true, buffer->get_insert()->get_iter().get_offset(), key->string);
  
  return true;
}

// If the keypress is a backspace, add the action or append it to the current
// action. This function receives the current text buffer the beginning 
// iterator of the insert, the key that was pressed, the action manager and 
// the environment settings object.
bool key_pressed_backspace (Glib::RefPtr<Gtk::TextBuffer> buffer, Gtk::TextIter begin, 
                            GdkEventKey *key, action_manager& aman, environment_settings env)
{
  Gtk::TextIter end;
  
  flip_insert_selection_bounds(buffer);
  begin = buffer->get_insert()->get_iter();
  end = buffer->get_selection_bound()->get_iter();
  
  // Check if there is selected text
  if (begin != end) 
  {
    aman.add_new_action(false, begin.get_offset(), buffer->get_text(begin, end, true));
    buffer->erase(begin, end);
  } 
  // If there is no selected text ...
  else 
  {
    // Update the action or create a new one
    if (!aman.get_current_action_add_or_del() &&
        aman.get_current_action_offset() == buffer->get_insert()->get_iter().get_offset())
      aman.add_to_begin_current_action(buffer->get_text(--begin, end, true));
    else
      aman.add_new_action(false, begin.get_offset(), buffer->get_text(--begin, end, true));
    buffer->erase(begin, end);
  }
    
  return true;
} 

// If the keypress is a delete, add the action or append it to the current
// action. This function receives the current text buffer the beginning 
// iterator of the insert, the key that was pressed, the action manager and 
// the environment settings object.
bool key_pressed_delete (Glib::RefPtr<Gtk::TextBuffer> buffer, Gtk::TextIter begin, 
                         GdkEventKey *key, action_manager& aman, environment_settings env)
{
  Gtk::TextIter end;
  
  flip_insert_selection_bounds(buffer);
  begin = buffer->get_insert()->get_iter();
  end = buffer->get_selection_bound()->get_iter();
  
  // Check if there is selected text
  if (begin != end) 
  {
    aman.add_new_action(false, begin.get_offset(), buffer->get_text(begin, end, true));
    buffer->erase(begin, end);
  } 
  // If there is no selected text...
  else {
    // Add to the current action or create a new one
    if (!aman.get_current_action_add_or_del() &&
        aman.get_current_action_offset() == buffer->get_insert()->get_iter().get_offset())
      aman.add_to_end_current_action(false, buffer->get_text(begin, ++end, true));
    else
      aman.add_new_action(false, begin.get_offset(), buffer->get_text(begin, ++end, true));
    buffer->erase(begin, end);
  }
    
  return true;
} 

// If the keypress is a tab, add the action. This function receives the current 
// text buffer the beginning iterator of the insert, the key that was pressed, 
// the action manager and the environment settings object. Tabs can be stored
// as a true tab or a number of spaces, which is specified by the user.
bool key_pressed_tab (Glib::RefPtr<Gtk::TextBuffer> buffer, Gtk::TextIter begin, 
                      GdkEventKey *key, action_manager& aman, environment_settings env)
{
  Gtk::TextIter end;
  
  flip_insert_selection_bounds(buffer);
  begin = buffer->get_insert()->get_iter();
  end = buffer->get_selection_bound()->get_iter();
  
  // Check if there is selected text
  if (begin != end) 
  {
    aman.add_new_action(false, begin.get_offset(), buffer->get_text(begin, end, true));
    buffer->erase(begin, end);
  }
  
  // Calculate the spaces for a tab
  Glib::ustring spaces;
  
  if (env.get_replace_tabs())
    for (int i=0; i<env.get_tabsize(); i++)
      spaces += " ";
  else
    spaces = "\t";
  
  // Insert spaces & create a new action
  buffer->insert_at_cursor(spaces);
  aman.add_new_action(true, buffer->get_insert()->get_iter().get_offset(), spaces);
     
  return true;
}

// If the keypress is a return, add the action. This function receives the current 
// text buffer the beginning iterator of the insert, the key that was pressed, 
// the action manager and the environment settings object. This function also
// handles autoindentation so it is indented the number of spaces as the previous
// line plus one more tab or set of spaces if it is preceded by a bracket.
bool key_pressed_return (Glib::RefPtr<Gtk::TextBuffer> buffer, Gtk::TextIter begin, 
                         GdkEventKey *key, action_manager& aman, environment_settings env)
{
  Gtk::TextIter end;
  
  flip_insert_selection_bounds(buffer);
  begin = buffer->get_insert()->get_iter();
  end = buffer->get_selection_bound()->get_iter();
  
  // Check if there is selected text
  if (begin != end) 
  {
    aman.add_new_action(false, begin.get_offset(), buffer->get_text(begin, end, true));
    buffer->erase(begin, end);
  }
    
  // Get current line & get iter at beginning of that line
  begin = buffer->get_insert()->get_iter();
  int line = begin.get_line();
  Glib::ustring spaces;
  begin = buffer->get_iter_at_line(line);
  end = begin;
    
  // Compute the number of spaces to insert
  char ch = end.get_char();
  while (g_unichar_isspace(ch) && ch != '\n')
  {
    if (!end.forward_char())
      break;
    ch = end.get_char();
  }
  spaces = "\n" + begin.get_slice(end);
    
  // Autoindent for '{'
  end.forward_to_line_end();
  end--;
  ch = end.get_char();
  if (ch == '{')
    for (int i=0; i<env.get_tabsize(); i++)
      spaces += ' ';
   
  // Insert auto-indentation
  buffer->insert_at_cursor(spaces);
  aman.add_new_action(true, buffer->get_insert()->get_iter().get_offset(), spaces);
    
  return true;
}
