/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "notebookmanager.h"
#include "utils.h"
#include "syntax.h"

// Notebook manager constructor that creates the notebook and sets its
// properties. It also sets the number of tabs to zero
notebook_manager::notebook_manager()
{
  // Create the notebook & set its settings
	notebook = new Gtk::Notebook();
	notebook->set_show_tabs(true);
  notebook->set_tab_pos(Gtk::POS_TOP);
  notebook->set_scrollable(true);
	notebook->show();
  notebook->signal_event_after().connect(sigc::mem_fun(*this, 
            &notebook_manager::notebook_event));
	tab_count = 0;
}

// Catch all events of the notebook so that the focus of the textview can
// be restored when a notebook tab is clicked
void notebook_manager::notebook_event(GdkEvent* ev)
{
  if (ev->type == GDK_BUTTON_PRESS)
  {
    tabs[notebook->get_current_page()].textview->grab_focus();
  }
}

// This function allows the main openldev class direct access to
// the notebook widget
Gtk::Notebook* notebook_manager::get_notebook()
{
	return notebook;
}

// Add a tab to the notebook. It receives the filename that is to be opened
// and the environment settings object. A new tab object is created if the
// file is not already open. The signals are then connected to the widgets and
// the fonts are set to the line numbering & TextView
bool notebook_manager::add_tab(Glib::ustring filename, environment_settings env)
{
  notebook_tab tab;
  tab.fn = filename;
  
  // Parse the filename to remove directory structure
  while (filename.find('/',0) != string::npos)
    filename.erase(0,filename.find('/',0)+1);

  // Check to make sure file is not already open
  for (int i=0; i<tab_count; i++)
  {    
    if (tab.fn == tabs[i].fn)
    {
      notebook->set_current_page(i);
      return false;
    }
  }

  // Create temporary tab & initialize the objects
	tab.initialize_objects(filename);
	tab.tab_id = tab_count;

  // Add the tab to the notebook and notebook_tab's deque
	tabs.push_back(tab);
	notebook->append_page(*tabs[tab_count].hbox2, *tabs[tab_count].hbox);
  
  // Set all of the signal commands
	tabs[tab_count].button->signal_clicked().connect(sigc::bind<Glib::ustring>(sigc::mem_fun(*this, 
          &notebook_manager::delete_tab), tabs[tab_count].fn));
  tabs[tab_count].scrolled_win->get_vadjustment()->signal_changed().connect(sigc::mem_fun(*this,
          &notebook_manager::sync_scrollbars));
   
  // Set the font and tab size of the textview  
  Glib::ustring font_description = env.get_font_face() + " " + env.get_font_size();
  Pango::FontDescription fd(font_description);  
  tabs[tab_count].textview->modify_font(fd);
  tabs[tab_count].line_numbers->modify_font(fd);
  tabs[tab_count].textview->set_tabs(get_tab_array(fd, env, tabs[tab_count].textview));
  
  // Do final initialization
  notebook->set_current_page(tab_count);
	tab_count++;
  return true;
}

// Revert to the saved version of the file. The file must be set as unmodified,
// then the file is reread & restored to its original version.
void notebook_manager::revert_to_saved()
{
  mark_current_tab_modified(false);
  
  int current = notebook->get_current_page();
  Glib::ustring text = read_file(tabs[current].fn);
  
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = Gtk::TextBuffer::create();
  text_buffer->set_text(text);
  tabs[current].textview->set_buffer(text_buffer);
}

// Delete the currently selected tab. First the user must confirm a closing of
// the file if it is modified. Then find the close button that was clicked,
// remove the tab, and clean up after the tab. It receives the full filename
// of the file that is going to be closed.
void notebook_manager::delete_tab(Glib::ustring fn)
{
  if (tabs[notebook->get_current_page()].label->get_text().find('*', 0) != string::npos)
    if (!question("The file you are closing is not saved.\nAre you sure you want to close?"))
      return;
  
  // Find file button that was clicked & get buffer
  int tab = 0;
  while (fn != tabs[tab].fn)
    tab++;
  Glib::RefPtr<Gtk::TextBuffer> text_buffer = tabs[tab].textview->get_buffer();
  
  // Remove the tab
	deque<notebook_tab>::iterator itr = tabs.begin();
	notebook->remove_page(tab);
	
  // Move the iter to the current tab
	for (int i=0; i < tab; i++)
		itr++;
	
  // Remove the tab objects & information
  tab_count--;
	tabs.erase(itr);
  
  // Set the current page to the previous one if there is a tab open
  if (tab_count != 0)
    notebook->set_current_page(tab_count);
}

// Return whether there is currently a file open. This will prevent a
// segmentation fault when compiling
bool notebook_manager::get_file_open()
{
  if (notebook->get_n_pages() > 0)
    return true;
  return false;
}

// Save all files in the tabs. Loop through all of the files and set them as
// unmodified. Then write the text from the buffer to the tab's full filename.
void notebook_manager::save_all_files()
{
  for (int i=0; i<notebook->get_n_pages(); i++)
  {    
    // Set the label as unmodified
    Glib::ustring text = tabs[i].label->get_text();
    if (text.find('*', 0) != string::npos)
      text.erase(text.find('*', 0), 1);
    tabs[i].label->set_text(text);
    
    // Save the file irregardless of whether it is modified or not
    write_file(tabs[i].fn, tabs[i].textview->get_buffer()->get_text(true));
  }
}

// Destroy everything in the notebook manager. This will remove all of the
// pages until the there are no more left.
void notebook_manager::erase_all_tabs()
{
  // Remove all of the tabs and tab information
  tabs.erase(tabs.begin(), tabs.end());
  while (notebook->get_n_pages() != 0)
    notebook->remove_page(0);
  tab_count = 0;
}

// Return the current textview so the main openldev class can directly access
// it. If there are no files open, then return a brand-spanking new textview
// that can just be trashed.
Gtk::TextView* notebook_manager::get_current_textview()
{
  int current = notebook->get_current_page();

  // Return the current textview
  if (tab_count != 0)
    return tabs[current].textview;
  // Return an empty textview if there are none
  else
  {
    Gtk::TextView *textview = new Gtk::TextView();
    return textview;
  }
}

// Return the actionmanager of the current tab by reference so it can be
// edited by the main openldev class
action_manager& notebook_manager::get_aman()
{
  return tabs[notebook->get_current_page()].aman;
}

// Return the current tab so that it can be edited by the main openldev class
int notebook_manager::get_current_tab()
{
  return notebook->get_current_page();
}

// Mark the current file as modified or unmodified. It receives whether it should
// be marked as modified or unmodified. Modified is shown by adding an asterick
// to the end of the short filename in the tab header
void notebook_manager::mark_current_tab_modified(bool modified)
{
  int current = notebook->get_current_page();
  Glib::ustring text = tabs[current].label->get_text();
  
  // If the file was modified
  if (modified)
  {
    if (text.find('*', 0) == string::npos)
      text += "*";
  }
  // If it was not modified
  else 
  {
    if (text.find('*', 0) != string::npos)
      text.erase(text.find('*', 0), 1);
  }
  
  tabs[current].label->set_text(text);
}


// Modify the font of each textview & line numbering. This receives a font
// description and is used when the user changes their default font.
void notebook_manager::modify_fonts(Pango::FontDescription fd)
{
  // Foreach tab, modify the fonts
  for (int i=0; i<tab_count; i++)
  {
    tabs[i].textview->modify_font(fd);
    tabs[i].line_numbers->modify_font(fd);
  }
}

// Scroll to the current cursor position. This is used for keypresses, find
// commands and goto line commands. It is needed for keypresses because the line
// numbering messes up the default properties of the TextView such as scrolling
// to the cursor when it reaches the end of the screen.
void notebook_manager::scroll_to_mark()
{
  int current = notebook->get_current_page();
  tabs[current].textview->scroll_to(tabs[current].textview->get_buffer()->get_insert(), 0.0);
}

// Get the filename of the current tab. This returns the full filename.
Glib::ustring notebook_manager::get_current_filename()
{
  return tabs[notebook->get_current_page()].fn;
}

// Get the short filename of the current tab. The asterick must be removed from
// the end of modified file names.
Glib::ustring notebook_manager::get_current_short_filename()
{
  int current = notebook->get_current_page();
  Glib::ustring fn = tabs[current].label->get_text();
  if (fn.find("*", 0) != string::npos)
    fn.erase(fn.find("*", 0), 1);
  return fn;
}

// Create the line numbering for a file. This gets the number of lines & manually
// creates the line numbering from one to the maximum number of lines.
void notebook_manager::create_line_numbers()
{
  int current = notebook->get_current_page();
  int lines = tabs[current].textview->get_buffer()->get_line_count()+1;
  Glib::ustring text;
  
  // Loop through the lines & add each line number to the string
  for (int i=1; i<lines; i++)
  {
    gchar number[4];
    sprintf(number, "%d", i);
    text += Glib::ustring(number) + "\n";
  }
  
  // Erase the last endl and then set the line numbers text
  text.erase(text.length()-1, 1);
  tabs[current].line_numbers->set_text(text);
  tabs[current].current_line_count = lines - 1;
}

// Update the line numbering. If the number of lines are not the same as the
// current display, the lines are recreated.
void notebook_manager::update_line_numbers()
{
  int current = notebook->get_current_page();
  int lines = tabs[current].textview->get_buffer()->get_line_count();
  
  // Compare the true number of lines and the shown number of lines
  if (tabs[current].current_line_count != lines)
    create_line_numbers();
}

// Show the line numbers or hide them. This receives whether the line numbering
// should be shown or hidden. Upon hide or show, both the line numbering label
// and scrolled window must be hidden or shown.
void notebook_manager::show_line_numbers(bool show_or_hide)
{
  for (int i=0; i<tab_count; i++)
  {
    if (show_or_hide)
    {
      tabs[i].line_numbers->show();
      tabs[i].swin->show();
    }
    else
    {
      tabs[i].line_numbers->hide();
      tabs[i].swin->hide();
    }
  }
}

// This makes sure the line numbering scrolled window and the TextView scrolled
// window keep the same adjustments. Otherwise, line numbering would be pointless.
void notebook_manager::sync_scrollbars()
{
  int current = notebook->get_current_page();
  tabs[notebook->get_current_page()].swin->set_vadjustment(tabs[current].scrolled_win->get_vadjustment());
}
