/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"
#include "syntax.h"
#include "utils.h"
#include "keypress.h"
#include "variables.h"

#include <gtkmm/stock.h>
#include <gtkmm/imagemenuitem.h>
#include <gtkmm/image.h>
#include <gdkmm/screen.h>
#include <gtkmm/iconfactory.h>
#include <gdkmm/event.h>
#include <sys/stat.h>
#include <pangomm/tabarray.h>

// This constructor sets the default properties for the OpenLDev window, creates
// the settings file if it does not exist and loads it if it does. Editor
// properties are then set & the last opened file is loaded if applicable.
openldev::openldev()
{  
  // Set properties of main window
  set_title("OpenLDev v" + Glib::ustring(VERSION));
  set_border_width(0);
  
  // Make directories & files if they do not exist
  Glib::ustring dir = Glib::get_home_dir();
  dir += "/.openldev/";
  mkdir(dir.c_str(), 0777);
  
  dir += "openldev.settings";
  if (!file_test(dir, Glib::FILE_TEST_EXISTS)) 
    env.create_new_settings_file();
  else 
    env.load_settings_file();
  
  // Call functions to add main window widgets & show them all
  set_default_size(env.get_width(), env.get_height());
  screen.set_toolbar(true);
  screen.set_fileviewer(true);
  screen.set_statusbar(true);
  screen.set_buildbox(true);
  initialize_objects();
  
  // Load the last project or just set the directory to the home directory
  if (env.get_lastproject() != "EMPTY" && env.get_autoload())
    open_project(env.get_lastproject());
}

// Create all of the objects needed by the main openldev window. Then call the
// functions to populate the find toolbar and goto line tolbar. Lastly, connect
// the signals to their widgets.
void openldev::initialize_objects()
{
  vpaned = new Gtk::VPaned();
  hpaned = new Gtk::HPaned();
  vbox = new Gtk::VBox(false, 0);
  hbox = new Gtk::HBox(false, 0);
  status_bar = new Gtk::Statusbar();
  status_bar2 = new Gtk::Statusbar();
  scrolled_win = new Gtk::ScrolledWindow();
  scrolled_win2 = new Gtk::ScrolledWindow();
  tree_view = new Gtk::TreeView();
  build_box = new Gtk::TreeView();
  noteman = new notebook_manager();
  
  // Add all of the widgets & show them
  populate_find_toolbar();
  populate_goto_line_toolbar();
  populate_window();  
  show_all_children();
  image_error->hide();
  label_error->hide();
  image_error2->hide();
  label_error2->hide();
  on_edit_goto_line();
  on_edit_find();
  
  // Connect widgets to their respective signals
  signal_event_after().connect(sigc::mem_fun(this, &openldev::on_resize));
  hpaned->signal_event_after().connect(sigc::mem_fun(this, &openldev::on_resize_pane));
  vpaned->signal_event_after().connect(sigc::mem_fun(this, &openldev::on_resize_pane));
}

// Set the properties of all of the widgets found in the main openldev window.
// This includes loading the menu, toolbar, project files TreeView, notebook
// manager, build output ListStore and the status bar
void openldev::populate_window()
{
  { // Create menu actions and icons
    create_icon_factory();
    create_action_group();
    create_uimanager_string();
  
    // Create the UIManager
    uimanager = Gtk::UIManager::create();
    uimanager->insert_action_group(actgroup);
    add_accel_group(uimanager->get_accel_group());
    uimanager->add_ui_from_string(ui_info);
    
    // Create toolbar UIManager
    uimanager_toolbar = Gtk::UIManager::create();
    uimanager_toolbar->insert_action_group(actgroup);
    add_accel_group(uimanager_toolbar->get_accel_group());
    uimanager_toolbar->add_ui_from_string(ui_toolbar);
  
    // Create & add menubar/toolbar/popupmenus to the first vbox
    Menubar = uimanager->get_widget("/MenuBar");
    toolbar = dynamic_cast<Gtk::Toolbar*>(uimanager_toolbar->get_widget("/ToolBar"));
    toolbar->set_toolbar_style(Gtk::TOOLBAR_ICONS);
    toolbar->set_show_arrow(false);
    menu_source_browser = dynamic_cast<Gtk::Menu*>(uimanager->get_widget("/TreeViewMenu")); 
    ui_info = "\0";
    ui_toolbar = "\0";
  }
  
  // Add menu & toolbar to main vbox
  vbox->pack_start(*Menubar, Gtk::PACK_SHRINK);  
  vbox->pack_start(*toolbar, Gtk::PACK_SHRINK);
  
  { // Create project files tree view
    Glib::RefPtr<Gtk::TreeStore> model;
    ModelColumns columns;
    scrolled_win->add(*tree_view);
    scrolled_win->set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS);

    // Create the Tree model:
    model = Gtk::TreeStore::create(columns);
    tree_view->set_model(model);

    // Fill the TreeView's model
    Gtk::TreeModel::Row row = *(model->append());
    row[columns.col_text] = "Sources";
    row = *(model->append());
    row[columns.col_text] = "Headers";
    row = *(model->append());
    row[columns.col_text] = "Other Files";

    // Add the TreeView's view columns:
    tree_view->append_column("Project Files", columns.col_text);
  }
  
  scrolled_win->set_size_request(env.get_treeview_width(), 0);
  tree_view->signal_button_press_event().connect_notify(sigc::mem_fun(*this, 
      &openldev::on_button_press_event_treeview));
  tree_view->get_selection()->select(tree_view->get_model()->get_iter("0"));

  // Create the build_box to add compiler output to
  scrolled_win2->add(*build_box);
  scrolled_win2->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
   
  // Fill the ListStore's model
  Glib::RefPtr<Gtk::ListStore> build_model;
  ModelColumns build_columns;
  build_model = Gtk::ListStore::create(build_columns);
  build_box->set_model(build_model);
  build_box->append_column("Compiler Output", build_columns.col_text);
  build_box->signal_button_press_event().connect_notify(sigc::mem_fun(*this,
      &openldev::build_box_selected));

  hpaned->pack1(*scrolled_win, Gtk::PACK_SHRINK, 3);
  hpaned->pack2(*noteman->get_notebook());
  vpaned->pack1(*hpaned);
  vpaned->pack2(*scrolled_win2, Gtk::PACK_SHRINK, 3);

  // Status bar handling
  status_bar->set_has_resize_grip(false);
  status_bar->set_size_request(env.get_treeview_width(), -1);
  hbox->pack_start(*status_bar, Gtk::PACK_SHRINK);
  hbox->pack_start(*status_bar2);
  hbox->set_spacing(5);

  // Add scrolled_win3 to the second vbox and finish off with the statusbar
  scrolled_win2->set_size_request(0, env.get_buildbox_height());
  vbox->pack_start(*vpaned);
  vbox->pack_start(*hbox_goto, Gtk::PACK_SHRINK);
  vbox->pack_start(*hbox_find, Gtk::PACK_SHRINK);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK); 

  // Add v' & h' boxes to the main window
  add(*vbox);
}

// Quit the program
void openldev::on_file_quit() 
{ 
  hide(); 
}

// When the window is resized, this checks the width and height and stores
// them so the same size can be restored on restart.
void openldev::on_resize(GdkEvent *ev)
{
  if (ev->type == GDK_CONFIGURE)
  {
    // Get the current height & width and store them
    int width=0, height=0;
    this->get_size(width, height);
    env.set_width(width);
    env.set_height(height);
    env.write_settings_file();
  }
}

// When one of the panes is resized, this checks the width and height of each
// pane so the same sizes can be restored on restart.
void openldev::on_resize_pane(GdkEvent *ev)
{
  if (ev->type == GDK_MOTION_NOTIFY)
  {
    // Get the Project File & Output panes width/height & store them
    env.set_treeview_width(scrolled_win->get_width());
    env.set_buildbox_height(scrolled_win2->get_height());
    env.write_settings_file();
    status_bar->set_size_request(env.get_treeview_width(), -1);
  }
}

// Close project file. First call the close project funtion and then erase
// the project files treeview, save all of the files and erase all of the tabs.
void openldev::close_project()
{
  proj.close_project();
  populate_treeview();
  noteman->save_all_files();
  noteman->erase_all_tabs();
}

// Update the status bar. This function receives the iterator of the cursor
// position and the mark at that point. It then calculates the line and column
// position and adds them with the project name an version to the status bar.
// The project and version are labelled as 'N/A' if a project is not open.
void openldev::update_status_bar (Gtk::TextIter iter, Glib::RefPtr<Gtk::TextBuffer::Mark> mark)
{
  char vertical[5], horizontal[5];
  g_snprintf(vertical, sizeof(vertical), "%d", noteman->get_current_textview()->get_buffer()->get_insert()->get_iter().get_line()+1);
  g_snprintf(horizontal, sizeof(horizontal), "%d", noteman->get_current_textview()->get_buffer()->get_insert()->get_iter().get_line_offset()+1);
       
  if (!proj.get_name().empty())
    status_bar2->push(Glib::ustring("Project: " + proj.get_name() + "       Version: " + proj.get_version() 
                      + "       Line: " + vertical + "       Column: " + horizontal), 0);
  else
    status_bar2->push(Glib::ustring(Glib::ustring("Project: None\tVersion: N/A\tLine: ") + vertical
                             + "       Column: " + horizontal), 0);
}

// Catch all of the keystrokes. If it is a letter, number, space, symbol,
// backspace, delete, tab or return, then process it. Otherwise return false.
// If processed, update the syntax highlighting, mark the file modified, scroll
// to the mark, update the line numbers and focus on the textview.
bool openldev::on_keystroke(GdkEventKey *key)
{
  bool key_is_completed = false;

  if (key->type == GDK_KEY_PRESS) 
  {    
    Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
    Gtk::TextIter iter = buffer->get_insert()->get_iter();
    
    // Call specific key press function
    if (isalnum(key->string[0]) || key->keyval == GDK_space)
      key_is_completed = key_pressed_alnum(buffer, iter, key, noteman->get_aman(), env);
    else if (key->keyval == GDK_BackSpace)
      key_is_completed = key_pressed_backspace(buffer, iter, key, noteman->get_aman(), env);
    else if (key->keyval == GDK_Delete || key->keyval == GDK_KP_Delete)
      key_is_completed = key_pressed_delete(buffer, iter, key, noteman->get_aman(), env);
    else if (key->keyval == GDK_KP_Tab || key->keyval == GDK_Tab)
      key_is_completed = key_pressed_tab(buffer, iter, key, noteman->get_aman(), env);
    else if (key->keyval == GDK_Return || key->keyval == GDK_Linefeed ||
             key->keyval == GDK_ISO_Enter || key->keyval == GDK_KP_Enter)
      key_is_completed = key_pressed_return(buffer, iter, key, noteman->get_aman(), env);
    else
    { // All of the keyboard symbols that aren't defined above
      switch(key->keyval)
      {
        case GDK_exclam: case GDK_quotedbl: case GDK_numbersign: case GDK_dollar:
        case GDK_percent: case GDK_ampersand: case GDK_quoteright: case GDK_parenleft: 
        case GDK_parenright: case GDK_asterisk: case GDK_plus: case GDK_comma: case GDK_minus: 
        case GDK_period: case GDK_slash: case GDK_colon: case GDK_semicolon: case GDK_less: 
        case GDK_equal: case GDK_greater: case GDK_question: case GDK_bracketleft: case GDK_backslash: 
        case GDK_bracketright: case GDK_asciicircum: case GDK_underscore: case GDK_grave: 
        case GDK_braceleft: case GDK_bar: case GDK_braceright: case GDK_asciitilde:
          key_is_completed = key_pressed_alnum(buffer, iter, key, noteman->get_aman(), env);
          break;
        default:
          break;          
      }
    }
    
    // If the key was processed already
    if (key_is_completed)
    {
      // Call syntax highlighting for the 1st time if filename is empty
      if (noteman->get_current_filename().empty()) 
      {
        set_syntax(buffer, true, true, env);
        screen.set_filename("EMPTY");
      }
      // Refresh syntax highlighting
      else if (key->keyval == GDK_slash)
        set_syntax(buffer, false, true, env);
      else
        set_syntax(buffer, false, false, env);
      
      noteman->mark_current_tab_modified(true);  
      noteman->scroll_to_mark();
      noteman->update_line_numbers();
      set_focus(*noteman->get_current_textview());
      return true;
    }
  }
  
  // If not processed yet, process the key
  return false;
}
