/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev_settings.h"
#include "project_info.h"
#include "customizetoolbar.h"
#include "environment.h"
#include "utils.h"

// Load the environment settings dialog and then store the changes that the user
// makes and rewrite the environment settings file. Then update the fonts.
void openldev::on_settings_env() 
{
  environment_dialog *dialog;
  dialog = new environment_dialog(env);
  int result = dialog->run();
  
  switch (result)
  {
    case Gtk::RESPONSE_OK: 
    {
      // Change properties by reference & destroy    
      env.set_terminal(dialog->get_terminal());
      env.set_font_face(dialog->get_font_face());
      env.set_font_size(dialog->get_font_size());
      env.set_replace_tabs(dialog->get_replace_tabs());
      env.set_error(dialog->get_error_color());
      env.set_warning(dialog->get_warning_color());
      env.set_syntax(0, dialog->get_variables());
      env.set_syntax(1, dialog->get_characters());
      env.set_syntax(2, dialog->get_strings());
      env.set_syntax(3, dialog->get_comments());
      env.set_syntax(4, dialog->get_keywords());
      env.set_syntax(5, dialog->get_preprocessor());
      env.set_autosave(static_cast<int>(dialog->get_asave_minutes()));
      env.set_tabsize(static_cast<int>(dialog->get_tab_size()));
      env.set_autoload(dialog->get_load_on_launch());
      env.write_settings_file();
      break;
    }
    default:
      break;
  }
  
  dialog->hide();
  
  // Reset fonts for all tabs
  Glib::ustring font_description = env.get_font_face();
  font_description += " " + env.get_font_size();
  Pango::FontDescription fd(font_description);  
  noteman->modify_fonts(fd);
}

// Load the filemanager and then store the changes to the sources, headers and
// other files and then write the new makefile and re-populate the treeview.
void openldev::on_settings_filemanager()
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    file_manager *dialog;
    dialog = new file_manager(proj.get_source_files(), proj.get_header_files(), 
                              proj.get_other_files(), proj.get_location());
    int result = dialog->run();
    
    // Catch when the dialog emits a close signal
    switch (result)
    {
      case (Gtk::RESPONSE_CLOSE): 
      {
        // Read & set the source files
        proj.set_source_files(dialog->get_source_files());
        proj.set_header_files(dialog->get_header_files());
        proj.set_other_files(dialog->get_other_files());
        populate_treeview();
        proj.write_source_makefile();        
        break; 
      }        
    }
    
    // Clean up after the function
    proj.write_project_file();
    dialog->hide();
  }
}

// If a project is open, load the project properties dialog and set the
// changes specified by the user. Also, rewrite the project compile flags to
// the src/Makefile.am
void openldev::on_settings_file_properties() 
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    project_properties_dialog *dialog;
    dialog = new project_properties_dialog(proj);
    int result = dialog->run();
    
    if (result == Gtk::RESPONSE_OK)
    {
      int begin, end;
      Glib::ustring text, makefile_location, commands;
      
      // Set project properties & save project file
      proj.set_author(dialog->get_author());
      proj.set_email(dialog->get_email());
      proj.set_homepage(dialog->get_homepage());
      proj.set_version(dialog->get_version());
      proj.write_project_file();
      
      // Set last project to new file in case project name has changed
      Glib::ustring filename = proj.get_location() + proj.get_name() + ".openldev";
      env.set_lastproject(filename);
      env.write_settings_file();
      
      // Create a string for the location of the src/Makefile.am
      makefile_location = proj.get_location() + "src/Makefile.am";
      
      // Read in the Makefile text
      text = read_file(makefile_location);
      
      // Find where the compiler flags are & erase them
      begin = text.find("FLAGS =", 0) + 8;
      end = text.find("bin_PROGRAMS", begin) - 1;
      text.erase(begin, end - begin);
      
      // Write first flag without a '\'
      commands = dialog->get_commands();
      text.insert(begin, commands);

      // Write new Makefile.am file
      write_file(makefile_location, text);
    }
    
    dialog->hide();
  }
}

// If a project is open, load the project information dialog.
void openldev::on_project_properties()
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    project_info *dialog;
    dialog = new project_info(proj);
    int result = dialog->run();
  
    switch (result)
    {
      default:
        break;
    }
  
    dialog->hide();
  }
}

// Load the toolbar customization dialog. Then get the new toolbar string &
// rewrite the environment settings dialog. Lastly, warn the user that the
// Changes will not take effect until they restart OpenLDev.
void openldev::on_settings_toolbar()
{
  customize_toolbar *dialog;
  dialog = new customize_toolbar(env, actgroup);
  int result = dialog->run();
  
  switch (result)
  {
    // Write the changes of the toolbar
    case -10:
    {
      env.set_toolbar(dialog->get_toolbar());
      env.write_settings_file();
      warning("You must restart OpenLDev for\nthe changes to take effect!");
      break;
    }
    default:
      break;
  }
  
  dialog->hide();
}
