/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project.h"
#include "utils.h"
#include <iostream>
#include <gtkmm/window.h>
#include <glib.h>
using namespace std;

Glib::ustring get_attrib(Glib::ustring attribute, Glib::ustring &text);

// Load the project file that is received by the function. If the file does
// not exist, false is returned and a message will be printed to the user.
// If it exists, each attribute is loaded.
bool project_settings::load_project_file (Glib::ustring filename) 
{ 
  Glib::ustring txt, text;
  Glib::FileTest filetest = Glib::FILE_TEST_EXISTS;
  
  if (!Glib::file_test(filename, filetest))
    return false;
    
  // Open project file
  text = read_file(filename);

  // Get all of the attributes
  name = get_attrib("<name>", text);
  author = get_attrib("<author>", text);
  location = get_attrib("<location>", text);
  email = get_attrib("<email>", text);
  url = get_attrib("<url>", text);
  version = get_attrib("<version>", text);
  lastfile = get_attrib("<lastfile>", text);
  sources = get_attrib("<sources>", text);
  headers = get_attrib("<headers>", text);
  other_files = get_attrib("<otherfiles>", text);
  txt = get_attrib("<language>", text);
  if (txt == "+") language = '+';
  else language = 'C';

  return true; 
}

// Write the project settings to the openldev project file.
void project_settings::write_project_file () 
{
  Glib::ustring odp, fn = location;
    
  // Create .ODP file text
  odp = "<name>" + name 
      + "*\n<author>" + author 
      + "*\n<location>" + location 
      + "*\n<email>"  + email 
      + "*\n<url>" + url 
      + "*\n<version>" + version 
      + "*\n<language>" + language   
      + "*\n<lastfile>" + lastfile 
      + "*\n<sources>" + sources 
      + "*\n<headers>" + headers 
      + "*\n<otherfiles>" + other_files + "*\n";
  
  fn += name + ".openldev";
  
  // Write the file
  write_file(fn, odp);
}

// Get the private properties of the project settings class
Glib::ustring project_settings::get_name()         { return name; }
Glib::ustring project_settings::get_author()       { return author; }
Glib::ustring project_settings::get_email()        { return email; }
Glib::ustring project_settings::get_url()          { return url; }
Glib::ustring project_settings::get_version()      { return version; }
Glib::ustring project_settings::get_source_files() { return sources; }
Glib::ustring project_settings::get_header_files() { return headers; }
Glib::ustring project_settings::get_other_files()  { return other_files; }
Glib::ustring project_settings::get_lastfile()     { return lastfile; }
Glib::ustring project_settings::get_location()     { return location; }
char project_settings::get_language()              { return language; }

// Get specific attribute from the text. The function receives the attribute
// and the text of the project file. It returns that text of the attribute.
Glib::ustring get_attrib(Glib::ustring attribute, Glib::ustring &text)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    return text.substr(start, len);
  }
  
  return ""; 
}

// Close the project by clearing all of the files, but first write the
// project file
void project_settings::close_project()
{
  write_project_file();
  set_source_files("");
  set_header_files("");
  set_other_files("");
  set_lastfile("");
  set_name("");
  set_author("");
  set_homepage("");
  set_email("");
  set_version("");
}

// Set variables in the project class
void project_settings::set_source_files(Glib::ustring files) { sources = files; }
void project_settings::set_header_files(Glib::ustring files) { headers = files; }
void project_settings::set_other_files(Glib::ustring files)  { other_files = files; }
void project_settings::set_lastfile(Glib::ustring file)      { lastfile = file; }
void project_settings::set_name(Glib::ustring str)           { name = str; }
void project_settings::set_author(Glib::ustring str)         { author = str; }
void project_settings::set_email(Glib::ustring str)          { email = str; }
void project_settings::set_homepage(Glib::ustring str)       { url = str; }
void project_settings::set_version(Glib::ustring str)        { version = str; }

// Sort the files received by the function alphabetically. They are passed by
// reference so you don't have to return the files. The files are parsed into
// a vector, sorted and then unparsed.
void sort_files(Glib::ustring &files)
{
  vector<Glib::ustring> names;
  Glib::ustring temp, str;
  int location;
    
  // Parse the files list
  location = files.find("/",0);
  while (location >= 0) 
  {
    names.push_back(files.substr(0,location));
    files.erase(0,location+1);
    location = files.find("/",0);
  }
    
  // Sort files
  vector<Glib::ustring>::iterator start = names.begin();
  vector<Glib::ustring>::iterator end = names.end();
  sort(start,end);
    
  // Un-parse files list
  for (unsigned int x=0; x<names.size(); x++) 
    files += names.at(x) + "/";
}

// Add file to the sources, headers or other files. This function receives
// the short filename of the file that is to be added.
void project_settings::add_file(Glib::ustring fn)
{
  // Add a source file
  if (fn.find(".c", 0) != string::npos)
  {
    sources += fn;
    sort_files(sources);
  }
  // Add a header file
  else if (fn.find(".h", 0) != string::npos)
  {
    headers += fn;
    sort_files(headers);
  }
  // Add an "other file"
  else
  {
    other_files += fn;
    sort_files(other_files);
  }
}

// Remove the file from the list. The function receives the file that is to be
// removed from the list.
void project_settings::remove_file(Glib::ustring fn)
{
  // Delete the chosen source, header or other file's name
  if (fn.find(".c", 0) != string::npos)
  {
    int start = sources.find(fn, 0);
    sources.erase(start, fn.length()+1);
  }
  else if (fn.find(".h", 0) != string::npos)
  {
    int start = headers.find(fn, 0);
    headers.erase(start, fn.length()+1);
  }
  else
  {
    int start = other_files.find(fn, 0);
    other_files.erase(start, fn.length()+1);
  }
}

// Write the src/Makefile.am with the new sources. It reads in the file text,
// removes the files list & then adds the new sources list to the file.
void project_settings::write_source_makefile()
{
  // Open the Makefile.am file & read in its text
  Glib::ustring fn = Glib::ustring(location) + "src/Makefile.am", temp, mfiles,
                text = read_file(fn);
        
  // Find & erase the current sources listing
  int loc1, loc2;
  loc1 = text.find("SOURCES = \\",0);
  loc2 = text.find("LDFLAGS",loc1);
  loc2 = text.rfind("\n",loc2);
  text.erase(loc1+11,loc2-loc1-11);
        
  // Add the source files to the string
  mfiles += "\n\t";
  temp = sources;
  while (!temp.empty())
  {
    loc2 = temp.find("/",0);
    mfiles += temp.substr(0,loc2) + "\\\n\t";
    temp.erase(0,loc2+1);
  }
        
  // Add the header files to the string
  temp = headers;
  while (!temp.empty())
  {
    loc2 = temp.find("/",0);
    mfiles += temp.substr(0,loc2) + "\\\n\t";
    temp.erase(0,loc2+1);
  }
        
  // Insert the files list into the Makefile.am text
  if (mfiles.length() > 5)
    mfiles.erase(mfiles.length()-3,3);
  mfiles += "\n";
  text.insert(loc1+11, mfiles);
        
  // Write the file back into itself
  write_file(fn, text); 
}
