/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "treeview.h"
#include "utils.h"

bool is_error_or_warning (Glib::ustring output);
int get_file_name_and_line (Glib::ustring &output);

// Open file selected by the tree view. This function gets the filename
// from the project files TreeView and loads it if the file selected is not
// one of the headers
void openldev::tree_view_selected()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = tree_view->get_selection();
  Gtk::TreeModel::iterator iter = selection->get_selected();
    
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    row->get_value(0, fn);
   
    // If the text is not a group title, open it
    if (fn != "Sources" && fn != "Headers" && fn != "Other Files")
    {
      Glib::ustring file = proj.get_location() + "src/" + fn;
      open_file(file);
    }
  }
}

// Add new file(s) to the project. The user can choose one or more files from
// the FileChooserDialog. They are then parsed and added to the project one
// at a time. The project settings file and the src/Makefile.am are then
// rewritten.
void openldev::tree_view_add_file()
{  
  Gtk::FileChooserDialog dialog("Choose File", Gtk::FILE_CHOOSER_ACTION_OPEN);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::ADD, Gtk::RESPONSE_OK);
  dialog.set_current_folder(Glib::ustring(proj.get_location() + "src/"));
  dialog.set_select_multiple(true);
  
  int result = dialog.run();
  switch (result)
  {
    // If OK is pressed
    case(Gtk::RESPONSE_OK):
    {
      int location;
      vector<Glib::ustring> filenames = dialog.get_filenames();
      
      // Parse the files & add them to the project
      for (unsigned int i=0; i<filenames.size(); i++)
      {
        location = filenames[i].find_last_of("/");
        while (location >= 0)
        {
          filenames[i].erase(0, location+1);
          location = filenames[i].find_last_of("/");
        }
        filenames[i] += "/";
        
        proj.add_file(filenames[i]);
      }
      
      // Create the new project file & src/Makefile.am
      populate_treeview();
      proj.write_project_file();
      proj.write_source_makefile();
      break;
    }
    default:
      break; 
  }

}

// Remove the currently selected file from the list. The project settings file 
// and the src/Makefile.am are then rewritten.
void openldev::tree_view_remove()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = tree_view->get_selection();
  Gtk::ListStore::iterator iter = selection->get_selected();
  
  if (iter)
  {
    // Remove the file if it is not a title
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring fn;
    row->get_value(0, fn);
    if ((fn != "Sources") && (fn != "Headers") && (fn != "Other Files"))
      proj.remove_file(fn);
  }
  
  // Refresh the list and rewrite the project file and src/Makefile.am
  populate_treeview();
  proj.write_project_file();
  proj.write_source_makefile();
}

// Populate or update the TreeView. The sources, headers and other files are
// parsed and added one at a time to their corresponding section in the project
// files TreeView. The branches are then expanded and the first item is selected
void openldev::populate_treeview()
{
  int start=0;
  Glib::ustring text = proj.get_source_files(), file;
  Gtk::TreeModel::Row child;
  tree_view->remove_all_columns();
  
  // Create project files tree view
  Glib::RefPtr<Gtk::TreeStore> model;
  ModelColumns columns;

  // Create the Tree model:
  model = Gtk::TreeStore::create(columns);
  tree_view->set_model(model);

  // Fill the TreeView's model
  Gtk::TreeModel::Row row_sources = *(model->append());
  row_sources[columns.col_text] = "Sources";
  Gtk::TreeModel::Row row_headers = *(model->append());
  row_headers[columns.col_text] = "Headers";
  Gtk::TreeModel::Row row_others = *(model->append());
  row_others[columns.col_text] = "Other Files";

  // Add the TreeView's view columns:
  tree_view->append_column("Project Files", columns.col_text);

  // Append the sources
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      child = *(model->append(row_sources.children()));
      child[columns.col_text] = text.substr(start, i-start);
      start = i+1;
    }
  }
  
  // Append the header files
  start=0;
  text = proj.get_header_files();
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      child = *(model->append(row_headers.children()));
      child[columns.col_text] = text.substr(start, i-start);
      start = i+1;
    }
  }
  
  // Append the other files
  start=0;
  text = proj.get_other_files();
  for (unsigned int i=0; i<text.length(); i++)
  {
    if (text[i] == '/') 
    {
      child = *(model->append(row_others.children()));
      child[columns.col_text] = text.substr(start, i-start);
      start = i+1;
    }
  }
  
  // Expand all of the tree to show files
  tree_view->expand_all();
  tree_view->get_selection()->select(tree_view->get_model()->get_iter("0"));
}

// Catch button press event & open popup menu or handle selection. This function
// receives the button event. If it is a right click, then the popup menu is
// shown. If it is a double click, the selected file is opened
void openldev::on_button_press_event_treeview(GdkEventButton *event)
{
  if((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
    menu_source_browser->popup(event->button, event->time);
  else if (event->type == GDK_2BUTTON_PRESS)
    tree_view_selected();
}

// Catch the build box selection & do action. This function receives the button
// event. If it is a double click, get the text of the selection and check if
// it is an error or warning. If it is, then the file containing the error or
// warning is opened. The cursor is then moved line of the error or warning
void openldev::build_box_selected(GdkEventButton *event)
{
  if (event->type == GDK_2BUTTON_PRESS)
  {
    Glib::RefPtr<Gtk::TreeSelection> selection = build_box->get_selection();
    Gtk::TreeModel::iterator iter = selection->get_selected();
    
    if (iter)
    {
      Gtk::TreeModel::Row row = *iter;
      Glib::ustring output, file;
      int linenumber;
      row->get_value(0, output);
      
      // It the output is an error or a warning..
      if (is_error_or_warning(output))
      {
        linenumber = get_file_name_and_line(output) - 1;
        
        // Open the file the error was found in
        open_file(proj.get_location() + "src/" + output);
        
        // Move the cursor to the line of the error & scroll to the cursor
        Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
        Gtk::TextIter iter = buffer->get_iter_at_line(linenumber);
        buffer->move_mark(buffer->get_insert(), iter);
        buffer->move_mark(buffer->get_selection_bound(), iter);
        noteman->scroll_to_mark();
        set_focus(*noteman->get_current_textview());
      }
    }
  }
}

// Check if the output is an error or warning. If it only has two colons in
// close proximity of eachother in the beginning, then it is an error
// or a warning.
bool is_error_or_warning (Glib::ustring output)
{
  // Find :'s (It is an error or warning if 2 :'s are found close together
  int location  = output.find(":", 0);
  int location2 = output.find(":", location+1);
  
  // Check if :'s were found & return if they were in proper format
  if (location >= 0)
    if (location2 >= 0 && (location2-location < 5))
      return true;
  
  return false;
}

// Get file name and line of the error or warning. This function parses the
// error or warning sent by the output string. Output is then converted into
// only the short filename containing the error & the line number of the
// error or warning is returned.
int get_file_name_and_line (Glib::ustring &output)
{
  Glib::ustring line;
  int location = output.find(":", 0);
  
  // Get the line number & delete it out so only filename is left
  line = output.substr(location+1, output.length()-location-1);
  output.erase(location, output.length()-location);
  
  // Erase all except for the line number
  location = line.find(":", 0);
  line.erase(location, line.length()-location);
  
  // Return the line number
  locale_from_utf8(line);
  return atol(line.c_str());
}
