/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "utils.h"
#include <gtkmm/dialog.h>
#include <gtkmm/box.h>

// Throw an error dialog with the specified message
void error (Glib::ustring message)
{
  Gtk::MessageDialog errordialog(message, false, Gtk::MESSAGE_ERROR, (Gtk::ButtonsType)(Gtk::BUTTONS_OK), true);
  errordialog.run();
}

// Throw a warning dialog with the specified message
void warning (Glib::ustring message)
{
  Gtk::MessageDialog errordialog(message, false, Gtk::MESSAGE_WARNING, (Gtk::ButtonsType)(Gtk::BUTTONS_OK), true);
  errordialog.run();
}

// Show a question dialog with the specified message. Return the answer the
// user choses. This function handles creating the dialog, setting the properties
// and then deleting it and returning the answer.
bool question (Glib::ustring question)
{
  Gtk::Dialog *dialog = new Gtk::Dialog();
  Gtk::Button *yes = new Gtk::Button(Gtk::StockID("gtk-yes"));
  Gtk::Button *no = new Gtk::Button(Gtk::StockID("gtk-no"));
  Gtk::Image *image = new Gtk::Image(Gtk::StockID("gtk-dialog-warning"), Gtk::IconSize(5));
  Gtk::Label *label = new Gtk::Label(question);
  Gtk::HBox *hbox = new Gtk::HBox(false, 10);
  
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  hbox->set_border_width(10);
  hbox->set_spacing(10);
  hbox->pack_start(*image, Gtk::PACK_SHRINK);
  hbox->pack_start(*label, Gtk::PACK_SHRINK);
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->pack_start(*hbox);
  dialog->get_vbox()->set_spacing(7);
  dialog->set_title("Question");
  dialog->set_modal(false);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(true);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(true);
  dialog->add_action_widget(*no, -9);
  dialog->add_action_widget(*yes, -8);
  
  yes->show();
  no->show();
  image->show();
  label->show();
  hbox->show();
  
  switch (dialog->run())
  {
    case -8:
      dialog->hide();
      return true;
      break;
    default:
      dialog->hide();
      return false;
      break;
  }
}

// Set the tab width for when the file contains tabs instead of spaces. This
// function receives the font description, environment settings object and the
// current textview. It returns the tabarray from the textview
Pango::TabArray &get_tab_array(Pango::FontDescription fd, environment_settings &env,
                               Gtk::TextView *text_view)
{
  Pango::TabArray *tab_array;
  Glib::RefPtr<Pango::Layout> layout;
	Glib::ustring tab;
	int i=0, width, height;

  // Add spaces for the number of spaces per tab
	while (i < env.get_tabsize()) 
  {
		tab += " ";
		i++;
	}

  // Create the pango layout & get the pixel width
	layout = text_view->create_pango_layout(tab);
  layout->set_font_description(fd);
  layout->get_pixel_size(width, height);
 
  // Create the tab array from the width
	tab_array = new Pango::TabArray(1, TRUE);
  tab_array->set_tab(0, Pango::TAB_LEFT, width);

  return *tab_array;
}

// Create new stock menu item. This function receives the icon factory, the
// name of the item, the id of the item, and the filename of the icon.
void add_new_stock_item(Glib::RefPtr<Gtk::IconFactory> icon_factory, Glib::ustring name,
                            Glib::ustring id, Glib::ustring filename)
{
  Gtk::IconSource icon_source;
  Gtk::IconSet icon;
  const Gtk::StockID stock_id(id);
  
  // Set the properties of the stock menu/toolbar item
  icon_source.set_icon_name(name);
  icon_source.set_filename(filename);
  icon.add_source(icon_source);
  icon_factory->add(stock_id, icon);
  Gtk::Stock::add(Gtk::StockItem(stock_id, name));
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
Glib::ustring read_file(Glib::ustring filename)
{
  Glib::FileTest filetest = Glib::FILE_TEST_EXISTS;
  if (!Glib::file_test(filename, filetest))
    return Glib::ustring("");
  
  Glib::ustring text;
  Glib::RefPtr<Glib::IOChannel> channel = Glib::IOChannel::create_from_file(filename, "r");
  channel->read_to_end(text);
  channel->close();
  
  return text;
}

// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void write_file(Glib::ustring filename, Glib::ustring text)
{
  Glib::RefPtr<Glib::IOChannel> channel = Glib::IOChannel::create_from_file(filename, "w");
  channel->write(text);
  channel->close();
}
